﻿/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: PointCloudToDistanceMap.cs 537 2012-04-04 15:35:42Z cr333 $
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VideoLib.OpenGL;
using VideoLib;
using VideoLib.Frames;
using OpenTK.Graphics.OpenGL;

namespace RGBZcamera.Library
{
	public class PointCloudToDistanceMap : Node, IOpenGLNode
	{
		public VideoLib.Frames.PixelFormat Format { get; set; }

		public PointCloudToDistanceMap()
		{
			Format = VideoLib.Frames.PixelFormat.Grey8;
			AddDefaultPins();
		}

		public override void Process()
		{
			// get input frame
			var frame = GetInputFrame("in");
			if (PassOnEndOfStreamFrame(frame)) return;
			var pointCloud = CastFrameTo<TextureFrame>(frame);

			// read out alpha component of point cloud (contains distance or zero for invalid points)
			pointCloud.Bind();
			var outputFrame = new BitmapFrame(pointCloud.Width, pointCloud.Height, Format) { SequenceNumber = pointCloud.SequenceNumber };
			GL.PixelStore(PixelStoreParameter.PackAlignment, 4);
			GL.GetTexImage(TextureTarget.Texture2D, 0, OpenTK.Graphics.OpenGL.PixelFormat.Alpha, Format.ToOpenGLPixelType(), outputFrame.Data);
			pointCloud.Unbind();
			pointCloud.RefDec();

			PushFrame(outputFrame);
		}
	}
}
