﻿/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: FlowAverager.frag 537 2012-04-04 15:35:42Z cr333 $
*/
#version 120
#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D current;
uniform sampler2D previous;
uniform sampler2D flow;

uniform float weight;

void main()
{
	vec4 flows = texture2D(flow, gl_FragCoord.xy / textureSize2D(flow, 0));
	vec2 coords = gl_FragCoord.xy - vec2(1, -1) * flows.xy;

	vec4 old = texture2D(previous, coords          / textureSize2D(previous, 0));
	vec4 new = texture2D(current,  gl_FragCoord.xy / textureSize2D(current,  0));
	
	//float w = 1; // no filtering (baseline)
	float w = weight; // constant weight
	//float w = mix(weight, 1, clamp(10 * abs(old.w - new.w), 0, 1)); // adaptive weight

	gl_FragColor = mix(old, new, w);
}