﻿// $Id: Utils.cs 436 2010-10-15 16:26:38Z cr333 $
using System;
using System.Collections.Generic;
using System.Text;
using LibAvWrap;
using VideoLib.Frames;

namespace VideoLib.LibAv
{
	/// <summary>Some useful extensions.</summary>
	public static class Utils
	{
		static Utils()
		{
			LibAvUtils.UsePrintfLogger();
		}

		/// <summary>The log level. The higher, the more verbose. Use 0 for no logging.</summary>
		public static int LogLevel { get { return LibAvUtils.LogLevel; } set { LibAvUtils.LogLevel = value; } }

		/// <summary>Converts libav's PixelFormat to BitmapFrame's FormatType.</summary>
		/// <param name="pixelFormat">Some PixelFormat.</param>
		/// <returns>The matching FormatType, or Unknown.</returns>
		internal static VideoLib.Frames.PixelFormat ToVideoLibPixelFormat(this LibAvWrap.PixelFormat pixelFormat)
		{
			switch (pixelFormat)
			{
				case LibAvWrap.PixelFormat.Gray8:  return VideoLib.Frames.PixelFormat.Grey8;
				case LibAvWrap.PixelFormat.Gray16: return VideoLib.Frames.PixelFormat.Grey16;  // needs testing
				case LibAvWrap.PixelFormat.BGR24:  return VideoLib.Frames.PixelFormat.BGR24;
				case LibAvWrap.PixelFormat.RGB24:  return VideoLib.Frames.PixelFormat.RGB24; // needs testing
				case LibAvWrap.PixelFormat.RGB32:  return VideoLib.Frames.PixelFormat.BGR32; // needs testing: unsure about swap
				case LibAvWrap.PixelFormat.BGR32:  return VideoLib.Frames.PixelFormat.RGB32; // needs testing: unsure about swap
				default: return VideoLib.Frames.PixelFormat.Unknown;
			}
		}

		/// <summary>Converts BitmapFrame's FormatType to libav's PixelFormat.</summary>
		/// <param name="formatType">Some FormatType.</param>
		/// <returns>The matching PixelFormat, or None.</returns>
		internal static LibAvWrap.PixelFormat ToLibAvPixelFormat(this VideoLib.Frames.PixelFormat formatType)
		{
			switch (formatType)
			{
				case VideoLib.Frames.PixelFormat.Grey8:  return LibAvWrap.PixelFormat.Gray8;
				case VideoLib.Frames.PixelFormat.Grey16: return LibAvWrap.PixelFormat.Gray16; // needs testing
				case VideoLib.Frames.PixelFormat.BGR24:  return LibAvWrap.PixelFormat.BGR24;
				case VideoLib.Frames.PixelFormat.RGB24:  return LibAvWrap.PixelFormat.RGB24; // needs testing
				case VideoLib.Frames.PixelFormat.BGR32:  return LibAvWrap.PixelFormat.RGB32; // needs testing: unsure about swap
				case VideoLib.Frames.PixelFormat.BGRA32: return LibAvWrap.PixelFormat.RGB32; // needs testing: unsure about swap
				case VideoLib.Frames.PixelFormat.RGB32:  return LibAvWrap.PixelFormat.BGR32; // needs testing: unsure about swap
				case VideoLib.Frames.PixelFormat.RGBA32: return LibAvWrap.PixelFormat.BGR32; // needs testing: unsure about swap
				default: return LibAvWrap.PixelFormat.None;
			}
		}

		/// <summary>Wraps a BitmapFrame into a LibAvFrame (if possible).</summary>
		/// <param name="bitmapFrame">A BitmapFrame.</param>
		/// <returns>A LibAvFrame.</returns>
		internal static LibAvFrame ToLibAvFrame(this BitmapFrame bitmapFrame)
		{
			// convert bitmap format
			LibAvWrap.PixelFormat format = bitmapFrame.Format.ToLibAvPixelFormat();

			// check for valid format
			if (format == LibAvWrap.PixelFormat.None)
				throw new NotImplementedException();

			// wrap input frame in a LibAvFrame
			return new LibAvFrame(bitmapFrame.Data, format, bitmapFrame.Width, bitmapFrame.Stride, bitmapFrame.Height);
		}

		/// <summary>Converts a LibAvFrame to a BitmapFrame (with invalid SequenceNumber).</summary>
		/// <param name="libAvFrame">A LibAvFrame</param>
		/// <returns>A BitmapFrame.</returns>
		internal static BitmapFrame ToBitmapFrame(this LibAvFrame libAvFrame)
		{
			// convert bitmap format
			VideoLib.Frames.PixelFormat format = libAvFrame.PixelFormat.ToVideoLibPixelFormat();

			// check for valid format
			if (format == VideoLib.Frames.PixelFormat.Unknown)
				throw new NotImplementedException();

			// prepare the BitmapFrame
			return new BitmapFrame(libAvFrame.Scan0, libAvFrame.Width, libAvFrame.Stride, libAvFrame.Height, format);
		}
	}
}
