/*
This source code is part of the Floating Textures test program, which
is a derivative of the complete Floating Textures program containing
the offline version with prereconstructed geometry and prerecorded
video. 
For more information on Floating Textures please have a look at the article

"Floating Textures" 
by M. Eisemann, B. De Decker, M. Magnor, 
P. Bekaert, E. de Aguiar, N. Ahmed, C. Theobalt, A. Sellent
Computer Graphics Forum (Proc. Eurographics EG'08), vol. 27, no. 2, April 2008 


Copyright (C) 2008 Martin Eisemann

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 3 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT 
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, see <http://www.gnu.org/licenses/>.
*/

/**
 * \file Derivatives1D5.frag
 *
 * \brief Calculates the derivatives of an image using a 1x5 filter
 */ 

// enable draw_buffers //
#extension GL_ARB_draw_buffers : enable


// uniforms //
uniform sampler2D source; // source image to be filtered
uniform vec2 dxy; // step direction and size
uniform vec2 dyx; // orthogonal step direction and size

uniform float mask[5]; // 1x5 filter values (must be normalized)


// forward declarations //
vec4 imageFilter1D5(float[5] mask, vec2 dxy, sampler2D source, vec2 texCoord);


void main() {
  vec2 texCoord = gl_TexCoord[0].xy;

  gl_FragData[0] = imageFilter1D5(mask, dxy, source, texCoord);
  gl_FragData[1] = imageFilter1D5(mask, dyx, source, texCoord);
}
 
