﻿// $Id: TextureBinding.cs 489 2010-11-09 18:45:40Z cr333 $
using OpenTK.Graphics.OpenGL;

namespace VideoLib.OpenGL
{
	/// <summary>Defines a texture binding between pin and uniform names, as well as texture unit.</summary>
	public struct TextureBinding
	{
// ---- main properties ---------------------------------------------------------------------------

		/// <summary>Gets or sets the pin name to take a texture from when binding it.</summary>
		/// <remarks>Can be null, in which case it is not bound, but still set as uniform and unbound.</remarks>
		public string Pin { get; set; }

		/// <summary>Gets or sets the uniform name that is set to the given texture unit.</summary>
		public string Uniform { get; set; }

		/// <summary>Gets or sets the texture unit this texture is bound to.</summary>
		public int Unit { get; set; }

//---- convenience properties ---------------------------------------------------------------------

		/// <summary>Sets both pin and uniform name to the same name.</summary>
		public string Name
		{
			set { Pin = value; Uniform = value; }
		}

		/// <summary>Gets or sets the OpenTK <see cref="TextureUnit"/>.</summary>
		public TextureUnit TextureUnit
		{
			get { return Unit + TextureUnit.Texture0; }
			set { Unit = value - TextureUnit.Texture0; }
		}

//---- methods ------------------------------------------------------------------------------------

		/// <summary>Returns a <see cref="System.String"/> that represents this instance.</summary>
		/// <returns>A <see cref="System.String"/> that represents this instance.</returns>
		public override string ToString()
		{
			if (Pin == null)
				return string.Format("{0} => '{1}'", Unit, Uniform);
			else
				return string.Format("{0} => '{1}' (Pin '{2}')", Unit, Uniform, Pin);
		}
	}
}
