// $Id: Enums.h 262 2010-07-28 13:25:26Z cr333 $
#pragma once

using namespace System;

#include "libMesaSR/definesSR.h"

namespace VideoLib { namespace SR4000 {

	[Flags] public enum class AcquireMode {
		// spelling as in libMesaSR, just CamelCased
		CorFixPtrn  = AM_COR_FIX_PTRN, //!< turns on fix pattern noise correction <b>this should always be enabled for good distance measurement</b>
		Median      = AM_MEDIAN,       //!< turns on a 3x3 median filter
		ToggleFrq   = AM_TOGGLE_FRQ,   //!< For sr3k: toggles each frame from 19 to 21 MHz  \intDoc{mode}
		ConvGray    = AM_CONV_GRAY,    //!< Converts the amplitude image by multiplying by the square of the distance, resulting in image more like a conventional camera.  
		SwAnf       = AM_SW_ANF,       //!< Turns on the 7x7 software adaptive neighborhood filter  \intDoc{mode}
		Reserved0   = AM_RESERVED0,    //!< was AM_SR3K_2TAP_PROC  \intDoc{mode}
		Reserved1   = AM_RESERVED1,    //!< was AM_SHORT_RANGE For sr4k: this flag results in more precise coordinate transformations for small distances(<1m) <b>works only for SR_CoordTrfFlt()</b>
		ConfMap     = AM_CONF_MAP,     //!< For sr4k: process a confidencemap. this map is accesssible with SR_GetImageList
		HwTrigger   = AM_HW_TRIGGER,   //!< For sr4k: Acquisition starts, when a Hardware Trigger is received (AM_SW_TRIGGER must also be set)
		SwTrigger   = AM_SW_TRIGGER,   //!< For sr4k: Light source is only turned on, when an image is requested
		DenoiseAnf  = AM_DENOISE_ANF,  //!< For sr4k: Turns on the 5x5 hardware adaptive neighborhood filter
		MedianCross = AM_MEDIANCROSS,  //!< turns on a 3x3 cross-median  filter (5 values)  \intDoc{mode}

		Undefined   = 0,
		Triggered   = HwTrigger | SwTrigger,
	};

	public enum class ImageType {
		Distance  = ImgEntry::IT_DISTANCE,  //!< distance image. Values between 0..0xffff (0xffff=maximal range which depends on modulation frequency)
		                                    //!< the distance image is affected by the #AcquireMode flags. For further information read the User Manual.
		Amplitude = ImgEntry::IT_AMPLITUDE, //!< amplitude image. For further information read the User Manual.
		Intensity = ImgEntry::IT_INTENSITY, //!< intensity image (only for SR2).
		Tap0      = ImgEntry::IT_TAP0,      //!< tap0 image. \intDoc{ImgType}
		Tap1      = ImgEntry::IT_TAP1,      //!< tap1 image. \intDoc{ImgType}
		Tap2      = ImgEntry::IT_TAP2,      //!< tap2 image. \intDoc{ImgType}
		Tap3      = ImgEntry::IT_TAP3,      //!< tap3 image. \intDoc{ImgType}
		SumDiff   = ImgEntry::IT_SUM_DIFF , //!< sum_diff image. (T0+T2)-(T1+T3) \intDoc{ImgType}
		ConfMap   = ImgEntry::IT_CONF_MAP,  //!< confidence map image. Value 0xffff is highest confidence.
		Undefined = ImgEntry::IT_UNDEFINED, //!< any other kind of image
		Last      = ImgEntry::IT_LAST       //!< useful last entry
	};

	public enum class DataType {
		None   = ImgEntry::DT_NONE,    //!< no data type
		UChae  = ImgEntry::DT_UCHAR,   //!< unsigned char data type
		Char   = ImgEntry::DT_CHAR,    //!< char data type
		UShort = ImgEntry::DT_USHORT,  //!< unsigned short data type
		Short  = ImgEntry::DT_SHORT,   //!< short data type
		UInt   = ImgEntry::DT_UINT,    //!< unsigned int data type
		Int    = ImgEntry::DT_INT,     //!< int data type
		Float  = ImgEntry::DT_FLOAT,   //!< float data type
		Double = ImgEntry::DT_DOUBLE   //!< double data type
	};

}} // closing namespace VideoLib.SR4000