﻿// $Id: Frame.cs 436 2010-10-15 16:26:38Z cr333 $
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VideoLib.Frames
{
	/// <summary>
	/// Abstract Frame class.
	/// </summary>
	public abstract class Frame : ICloneable
	{
		/// <summary>
		/// The sequence number of a frame. Use -1 for undefined.
		/// </summary>
		public int SequenceNumber { get; set; }

		/// <summary>
		/// Default constructor for undefined sequence number
		/// </summary>
		public Frame()
		{
			SequenceNumber = -1;
		}

		/// <summary>
		/// Construct a frame.
		/// </summary>
		/// <param name="sequenceNumber">Sequence number of this frame.</param>
		public Frame(int sequenceNumber)
		{
			SequenceNumber = sequenceNumber;
		}

		/// <summary>Copy this frame's contents.</summary>
		/// <remarks>All frames should be able to copy their contents to another Frame.</remarks>
		/// <param name="newFrame">The destination Frame where this Frame's data should be copied to.</param>
		public abstract void copyTo(ref Frame newFrame);

		#region ICloneable Members

		/// <summary>
		/// Creates a new object that is a copy of the current instance.
		/// </summary>
		public virtual object Clone()
		{
			Frame newFrame = null;
			copyTo(ref newFrame);
			return newFrame;
		}

		#endregion
	}

	/// <summary>
	/// This interface marks reference-counted frames.
	/// </summary>
	public interface IRefCountedFrame
	{
		/// <summary>Increments the reference count of this frame.</summary>
		void RefInc();

		/// <summary>Decrements the reference count of this frame.</summary>
		/// <remarks>If the reference count reaches zero, the memory is deallocated.</remarks>
		void RefDec();
	}

	/// <summary>
	/// This interface marks reference-counted CUDA frames.
	/// </summary>
	public interface ICudaFrame : IRefCountedFrame
	{
	}
}
